import Taro from '@tarojs/taro'
import Route from '@/class/Route'
import { APP_ID, APP_TYPE, APP_VERSION_CODE, VERSION_CODE } from '@/config'

class Http {
  get(url: string, data = {}, config: ConfigType = {}) {
    const { header = {} } = config

    data = wrapData(data)

    return new Promise((success, fail) => {
      const expParams: ConfigType = {
        header: {
          'content-type': 'application/x-www-form-urlencoded',
          ...header
        }
      }

      Taro.request({
        url,
        data,
        success,
        fail,
        ...expParams
      })
    })
  }

  post(url: string, data = {}, config: ConfigType = {}) {
    const { header = {} } = config

    data = wrapData(data)

    if ((data as DataType).baseInfo)
      data = createBaseInfoToFetchData(data)

    return new Promise((resolve, reject) => {
      const expParams: ConfigType = {
        header: {
          'content-type': 'application/x-www-form-urlencoded',
          ...header
        },
        method: 'POST'
      }

      Taro.request({
        url,
        data,
        success(res) {
          const { data, statusCode } = res

          if (statusCode !== 200) {
            reject(data)
            return
          }

          resolve(data)
        },
        fail(res) {
          reject(res)
        },
        ...expParams
      })
    })
  }
}

export function wrapData(params) {
  const systemInfo = Taro.$global.get('systemInfo')
  const launchOption = Taro.$global.get('launchOption')
  const isReturningUser = Taro.$global.get('isReturningUser')
  const userInfo = Taro.$global.get('userInfo') || {}
  const EXPERIMENT_CONFIG = Taro.getStorageSync('$EXPERIMENT_CONFIG')

  const route = new Route()

  const defaultInfo = {
    appId: APP_ID,
    appType: APP_TYPE,
    realAppType: APP_TYPE, // ？
    versionCode: VERSION_CODE,
    appVersionCode: APP_VERSION_CODE,
    token: userInfo.accessToken || '',
    loginUid: userInfo.uid || '',
    clientTimestamp: new Date().getTime(),
    pageSource: route.path,
    machineCode: Taro.$global.get('mid') || '',
    machineInfo: getMachineInfoFromSystem(),
    networkType: Taro.$network.networkType,
    network: Taro.$network.networkType,
    platform: systemInfo.platform,
    system: systemInfo.system,
    careModelStatus: systemInfo.easyMode,
    senceType: launchOption.scene,
    hotSenceType: launchOption.hotScene,
    abExpInfo: JSON.stringify(EXPERIMENT_CONFIG),
    sessionId: Taro.$global.get('sessionId'),
    subSessionId: Taro.$global.get('subSessionId'),
  }

  // 单独处理30天无回流
  if (isReturningUser) {
    params.extParams = JSON.stringify({
      ...params?.extParams,
      has30DayReturnCnt: isReturningUser
    })
  }

  return {
    ...defaultInfo,
    ...params
  }
}

export function createBaseInfoToFetchData(data) {
  const baseKey = ['token', 'loginUid', 'appType', 'machineCode', 'platform', 'machineInfo', 'networkType', 'pageSource', 'rootPageSource', 'clientTimestamp', 'sessionId', 'subSessionId', 'requestId', 'pageCategoryId', 'rootPageCategoryId', 'openType', 'shareDepth', 'eventId', 'videoReportMeta', 'rootPageTimestamp', 'returnId', 'system']

  return Object.keys(data).reduce((calc, key) => {
    if (baseKey.includes(key)) {
      calc.baseInfo[key] = data[key]
    } else {
      calc.params[key] = data[key]
    }

    return calc
  }, { baseInfo: {}, params: {} })
}

export function getMachineInfoFromSystem() {
  const systemInfo = Taro.$global.get('systemInfo')
  const keys = ['sdkVersion', 'brand', 'language', 'model', 'platform', 'system', 'version', 'screenHeight', 'screenWidth', 'pixelRatio', 'windowHeight', 'windowWidth']

  const result = keys.reduce((calc, key) => {
    // 适配其他小程序数据上报
    if (key === 'version')
      key = 'weChatVersion'

    if (key === 'SDKVersion')
      key = 'sdkVersion'

    calc[key] = systemInfo[key]

    return calc
  }, {})

  return JSON.stringify({
    softVersion: '', // TODO： appConfig.versionName
    ...result
  })
}

export default new Http()
